@@ -23,7 +23,6 @@
 
 #include <string.h>
 #include <stdlib.h>
-#include <guile/gh.h>
 #include <dirent.h>
 #include <gnome.h>
 #include <games-frame.h>
@@ -36,7 +35,7 @@
 #include "events.h"
 
 static GtkWidget *hint_dlg = NULL;
-static GtkWidget* seed_entry;
+static GtkTreeIter selected_iter;
 
 void show_game_over_dialog() {
   GtkWidget* dialog;
@@ -44,24 +43,28 @@ void show_game_over_dialog() {
 
   if (game_won)
     message = g_strdup_printf ("<b>%s\n\n%s</b>", _("Congratulations."),
-                                _("You won!!!"));
+                                _("You have won!!!"));
   else
-    message = g_strdup_printf ("<b>%s</b>", _("Game over."));
+    message = g_strdup_printf ("<b>%s</b>", _("There are no more moves."));
 
-  dialog = gtk_message_dialog_new (GTK_WINDOW (app),
-                                   GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_INFO,
-                                   GTK_BUTTONS_NONE,
-                                   message);
+  dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (app),
+					       GTK_DIALOG_DESTROY_WITH_PARENT,
+					       GTK_MESSAGE_INFO,
+					       GTK_BUTTONS_NONE,
+					       message);
 
   g_free (message);
-  gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label),
-                            TRUE);
+
+  if (!game_won)
+    gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+			    GTK_STOCK_UNDO, GTK_RESPONSE_REJECT,
+			    NULL);
+  /* FIXME: New and New Game are both used. One is a stock item, the
+   * other is more descriptive. */
   gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                           _("New Game"), GTK_RESPONSE_ACCEPT,
-                          GTK_STOCK_QUIT, GTK_RESPONSE_REJECT,
                           NULL);
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
 
   /* add a stock icon? */ 
   switch (gtk_dialog_run (GTK_DIALOG (dialog)))
@@ -75,12 +78,14 @@ void show_game_over_dialog() {
     case GTK_RESPONSE_REJECT:
       {
         gtk_widget_destroy (dialog);
-        gtk_widget_destroy (app);
-        gtk_main_quit ();
+	game_over = FALSE;
+	undo_callback ();
       }
       break;
     default:
      {
+       /* The player closed the dialog window, just ignore this. */
+       game_over = FALSE;
        gtk_widget_destroy (dialog);
      }
   }
@@ -93,16 +98,22 @@ static void select_rules(GtkTreeSelection *select, gpointer data)
 	GtkTreeModel *model;
         GtkTreeIter iter;
 
-	gtk_tree_selection_get_selected(select, &model, &iter);
-        g_free (filename);
-	gtk_tree_model_get(model, &iter, 1, &filename, -1);
+	if (gtk_tree_selection_get_selected(select, &model, &iter)) {
+	     g_free (filename);
+	     gtk_tree_model_get(model, &iter, 1, &filename, -1);
+	} else {
+	     g_free (filename);
+	     filename = NULL;
+	}
 }
 
 static void select_game (GtkWidget *app, gint response, gpointer data)
 {
   if(response == GTK_RESPONSE_OK) {
-    seed = atoi (gtk_entry_get_text (GTK_ENTRY (seed_entry)));
-    new_game (filename, &seed);
+       if (filename == NULL)
+	    return;
+
+       new_game (filename, NULL);
   }
 
   gtk_widget_hide(app);
@@ -116,28 +127,42 @@ select_double_click (GtkWidget *widget, GtkTreePath *path,
   select_game (GTK_WIDGET (dialog), GTK_RESPONSE_OK, NULL);
 }
 
-void show_select_game_dialog() 
+static void build_list (gchar * filename, GtkWidget * list)
+{
+  gchar *text;
+  GtkTreeIter iter;
+
+  if (g_utf8_collate (filename, "sol.scm") == 0)
+    return;
+
+  text = game_file_to_name (filename);
+  gtk_list_store_append (GTK_LIST_STORE (list), &iter);
+  gtk_list_store_set(GTK_LIST_STORE (list), 
+                     &iter, 0, text, 1,
+                     filename, -1);
+  if (g_utf8_collate(text,game_name) == 0) {
+    memcpy (&selected_iter, &iter, sizeof(GtkTreeIter));
+  }
+  g_free (text);
+}
+
+void show_select_game_dialog (void) 
 {
   static GtkWidget* dialog = NULL;
   static GtkListStore* list;
-  GtkListStore ** listp;
   static GtkWidget* list_view;
   static GtkTreeSelection* select;
   GtkWidget* scrolled_window;
   GtkTreeViewColumn* column;
   GtkCellRenderer* renderer;
   GtkTreeIter iter;
-  GtkTreeIter selected_iter;
   GtkTreePath * path;
-  
-  guint i;
-  gchar buf[20];
+  GamesFileList * files;
 
   if (waiting_for_mouse_up()) return;
 
   if (!dialog) {
 
-    GtkWidget* label;
     GtkWidget* hbox;
 
     dialog = gtk_dialog_new_with_buttons (_("Select Game"),
@@ -148,18 +173,6 @@ void show_select_game_dialog()
                                           NULL);
     gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
     hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
-    seed_entry = gtk_entry_new ();
-    label = gtk_label_new(_("Seed"));
-    
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 
-			GNOME_PAD_SMALL );
-    gtk_box_pack_end (GTK_BOX (hbox), seed_entry, FALSE, FALSE, 
-		      GNOME_PAD_SMALL );
-    
-    gtk_box_pack_end (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-		      hbox, FALSE, FALSE, GNOME_PAD_SMALL );
-    
-    hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
 
     list = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
     list_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(list));
@@ -200,18 +213,12 @@ void show_select_game_dialog()
     filename = NULL;
     selected_iter.stamp = 0;
     
-    for(i = 0; i < n_games; i++) {
-	    gchar *text;
-	    text = game_file_to_name (game_dents[i]->d_name);
-	    gtk_list_store_append (GTK_LIST_STORE (list), &iter);
-            gtk_list_store_set(GTK_LIST_STORE (list), 
-                               &iter, 0, text, 1,
-                               game_dents[i]->d_name, -1);
-            if (g_utf8_collate(text,game_name) == 0) {
-                    memcpy (&selected_iter, &iter, sizeof(GtkTreeIter));
-            }
-            g_free (text);
-    }
+    files = games_file_list_new ("*.scm", gamesdir, NULL);
+    games_file_list_transform_basename (files);
+ 
+    games_file_list_for_each (files, (GFunc) build_list, list);
+
+    g_free (files);
 
     gtk_dialog_set_default_response ( GTK_DIALOG (dialog), GTK_RESPONSE_OK );
 
@@ -228,11 +235,7 @@ void show_select_game_dialog()
       gtk_tree_selection_select_iter (select, &selected_iter);    
   }
   
-  g_snprintf (buf, sizeof (buf), "%d", seed);
-  gtk_entry_set_text (GTK_ENTRY (seed_entry), buf);
-
-  listp = &list;
-  if (gtk_tree_selection_get_selected (select, (GtkTreeModel **)listp, &iter)) {
+  if (gtk_tree_selection_get_selected (select, NULL, &iter)) {
     path = gtk_tree_model_get_path (GTK_TREE_MODEL(list), &iter);
     gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (list_view), path, NULL, TRUE, 0.5, 0.0);
     gtk_tree_path_free (path);
@@ -258,47 +261,40 @@ void show_hint_dialog()
     gmessage = g_strdup (_("The game is over.\nNo hints are available"));
   }  
   else {
-    SCM hint = gh_call0(game_data->hint_lambda);
+    SCM hint = cscmi_hint_lambda ();
 
-    if (!gh_scm2bool(hint)) {
+    if (!SCM_NFALSEP (hint)) {
       gmessage = g_strdup (_("This game does not have hint support yet."));
     }
     else {
-      switch (gh_scm2int(gh_car(hint))) {
+      switch (scm_num2int (SCM_CAR (hint), SCM_ARG1, NULL)) {
 
       case 0:
-	gmessage = g_strdup (gh_scm2newstr(gh_cadr(hint),NULL));
+	gmessage = g_strdup (SCM_STRING_CHARS (SCM_CADR(hint)));
 	break;
 
       case 1:
-	str1 = gh_scm2newstr(gh_cadr(hint),NULL);
-	str2 = gh_scm2newstr(gh_caddr(hint),NULL);
+	str1 = SCM_STRING_CHARS (SCM_CADR (hint));
+	str2 = SCM_STRING_CHARS (SCM_CADDR (hint));
 	gmessage = g_strdup_printf (_("Move %s onto %s."), str1, str2);
-	free (str1);
-	free (str2);
 	break;
 
       case 2:
-	str1 = gh_scm2newstr(gh_cadr(hint),NULL);
-	str2 = gh_scm2newstr(gh_caddr(hint),NULL);
+	str1 = SCM_STRING_CHARS (SCM_CADR (hint));
+	str2 = SCM_STRING_CHARS (SCM_CADDR (hint));
 	gmessage = g_strdup_printf (_("Move %s onto %s."), str1, str2);
-	free (str1);
-	free (str2);
 	break;
 
       case 3: /* This is deprecated (due to i18n issues) do not use.*/
-	str1 = gh_scm2newstr(gh_cadr(hint),NULL);
-	str2 = gh_scm2newstr(gh_caddr(hint),NULL);
+	str1 = SCM_STRING_CHARS (SCM_CADR (hint));
+	str2 = SCM_STRING_CHARS (SCM_CADDR (hint));
 	gmessage = g_strdup_printf (_("Move %s %s."), str1, str2);
-	free (str1);
-	free (str2);
         g_warning(_("This game uses a deprecated hint method (case 3).\nPlease file a bug at http://bugzilla.gnome.org including this message and\nthe name of the game you where playing (look in the title bar if you \naren't sure)."));
 	break;
 
       case 4:
-	str1 = gh_scm2newstr(gh_cadr(hint),NULL);
+	str1 = SCM_STRING_CHARS (SCM_CADR (hint));
 	gmessage = g_strdup_printf (_("You are searching for a %s."), str1);
-	free (str1);
 	break;
 
       default:
@@ -306,18 +302,17 @@ void show_hint_dialog()
 	break;
       }
     }
-	}
+  }
 
   if (hint_dlg) {
    gtk_widget_destroy (GTK_WIDGET (hint_dlg));
-	  
   }
 
   hint_dlg = gtk_message_dialog_new (GTK_WINDOW (app),
-	                                   GTK_DIALOG_DESTROY_WITH_PARENT,
-	                                   GTK_MESSAGE_INFO,
-	                                   GTK_BUTTONS_OK,
-	                                   gmessage);
+				     GTK_DIALOG_DESTROY_WITH_PARENT,
+				     GTK_MESSAGE_INFO,
+				     GTK_BUTTONS_OK,
+				     gmessage);
   if (hint_dlg) {
 	  g_signal_connect (GTK_OBJECT (hint_dlg),
 			      "destroy",
@@ -325,8 +320,8 @@ void show_hint_dialog()
 			      NULL);
   }
 
-	gtk_dialog_run(GTK_DIALOG(hint_dlg));
-	gtk_widget_destroy(hint_dlg);
+  gtk_dialog_run (GTK_DIALOG (hint_dlg));
+  gtk_widget_destroy (hint_dlg);
 
   g_free (gmessage);
 }
@@ -390,13 +385,13 @@ option_apply (GtkWidget *w, int response)
 		if ( response != GTK_RESPONSE_OK )
 			return;
 		
-    for(; opts != SCM_EOL; item = item->next, opts = gh_cdr (opts)) {
+    for(; opts != SCM_EOL; item = item->next, opts = SCM_CDR (opts)) {
       
-      SCM_SETCAR (gh_cdar(opts), 
-		gh_bool2scm (GTK_TOGGLE_BUTTON 
-			     (((GtkBoxChild *) item->data)->widget)->active));
+      SCM_SETCAR (SCM_CDAR(opts), 
+		  SCM_BOOL (GTK_TOGGLE_BUTTON 
+			    (((GtkBoxChild *) item->data)->widget)->active));
     }    
-    gh_call1(game_data->apply_options_lambda, options);
+    cscmi_apply_options_lambda (options);
 }
 
 static GtkWidget *
@@ -405,12 +400,11 @@ get_option_page (GtkWidget* option_dialog)
   SCM opts;
   GtkWidget* vbox = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
 
-  for(opts = options; opts != SCM_EOL; opts = gh_cdr (opts)) {
+  for(opts = options; opts != SCM_EOL; opts = SCM_CDR (opts)) {
     GtkWidget *toggle = 
-      gtk_check_button_new_with_label (_(gh_scm2newstr(gh_caar(opts), 
-						       NULL)));
+      gtk_check_button_new_with_label (_(SCM_STRING_CHARS (SCM_CAAR(opts))));
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), 
-				 gh_scm2bool(gh_cadar(opts)));
+				 SCM_NFALSEP(SCM_CADAR(opts)));
     gtk_box_pack_end (GTK_BOX (vbox), toggle, 
 		      FALSE, FALSE, GNOME_PAD_SMALL);
   }
@@ -421,9 +415,9 @@ void show_rules_options_dialog ()
 {
 	static GtkWidget *notebook = NULL;
   /* Need to call this every time as it might be garbage collected ! */
-  options = gh_call0(game_data->get_options_lambda);
+	options = cscmi_get_options_lambda ();
   
-  if (!option_dialog && gh_scm2bool(options)) {
+  if (!option_dialog && SCM_NFALSEP(options)) {
     option_dialog = gtk_dialog_new ();
     gtk_dialog_set_has_separator (GTK_DIALOG (option_dialog), FALSE);
 		notebook = gtk_notebook_new();
