@@ -26,9 +26,7 @@
 #include <sys/types.h>
 #include <string.h>
 #include <stdlib.h>
-#include <time.h>
 #include <libguile.h>
-#include <guile/gh.h>
 #include <ctype.h>
 #include "sol.h"
 #include "events.h"
@@ -55,6 +53,7 @@
  * Global Variables
  */
 GtkWidget        *app;
+GtkWidget        *vbox;
 GtkWidget        *playing_area;
 GtkWidget        *option_dialog = NULL;
 GdkGC            *draw_gc;
@@ -134,7 +133,7 @@ void make_title ()
 {
   char *title;
 
-  title = g_strdup_printf (_("AisleRiot: %s (%d)"), game_name, seed);
+  title = g_strdup_printf (_("AisleRiot: %s"), game_name);
 
   gtk_window_set_title (GTK_WINDOW (app), title); 
 
@@ -147,7 +146,7 @@ void eval_installed_file (char *file)
   char *relative;
   
   if (g_file_test (file, G_FILE_TEST_EXISTS)){
-    gh_eval_file (file);
+    scm_c_primitive_load (file);
     return;
   }
   
@@ -157,8 +156,8 @@ void eval_installed_file (char *file)
                                                   relative,
                                                   FALSE, NULL);
 
-  if (g_file_test(installed_filename, G_FILE_TEST_EXISTS)){
-    gh_eval_file (installed_filename);
+  if (g_file_test (installed_filename, G_FILE_TEST_EXISTS)){
+    scm_c_primitive_load (installed_filename);
   } else {
     char *message = g_strdup_printf (
          _("Aisleriot can't load the file: \n%s\n\n"
@@ -195,7 +194,7 @@ save_state (GnomeClient *client)
   return TRUE;
 }
 
-void new_game (gchar* file, guint *seedp )
+void new_game (gchar* file, guint *seedp)
 {
   SCM size;
 
@@ -245,22 +244,23 @@ void new_game (gchar* file, guint *seedp )
   set_score();
   timer_reset ();
 
-  size = gh_call0(game_data->start_game_lambda);
-  gh_eval_str ("(start-game)");
+  size = cscmi_start_game_lambda ();
+  scm_c_eval_string ("(start-game)");
 
-  set_geometry (gh_scm2double (gh_car (size)), gh_scm2double (gh_cadr (size)));
+  set_geometry (scm_num2double (SCM_CAR (size), 0, NULL),
+		scm_num2double (SCM_CADR (size), 0, NULL));
 
   /* It is possible for some games to not have any moves right from the
    * start. If this happens we redeal. */
-  if (!gh_scm2bool (gh_call0 (game_data->game_over_lambda))) {
-    new_game (file, seedp);
+  if (!SCM_NFALSEP (cscmi_game_over_lambda ())) {
+    new_game (file, NULL);
   } else {
     if (surface)
       refresh_screen();
     
     undo_set_sensitive (FALSE);
     redo_set_sensitive (FALSE);
-    
+
     game_over = FALSE;
     make_title();
   }
@@ -281,7 +281,7 @@ GtkWidget *time_value;
 static gint timer_cb ()
 {
   timeout = 3600;
-  gh_call0(game_data->timeout_lambda);
+  cscmi_timeout_lambda ();
   end_of_game_test();
   return 0;	
 }
@@ -331,8 +331,8 @@ static void create_sol_board ()
    * window size. */
   gtk_widget_set_size_request (playing_area, BOARD_MIN_WIDTH, 
 			       BOARD_MIN_HEIGHT);
-  
-  gnome_app_set_contents (GNOME_APP (app), playing_area);
+
+  gtk_box_pack_start (GTK_BOX (vbox), playing_area, TRUE, TRUE, 0);
   
   gtk_widget_realize (playing_area);
 
@@ -386,22 +386,25 @@ static void create_main_window ()
 
 gchar* start_game;
 
-static void main_prog(int argc, char *argv[])
+static void main_prog(void *closure, int argc, char *argv[])
 {
   GtkWidget *score_label, *time_label, *score_box, *status_bar;
-  
-  seed = time(NULL);
-  g_random_set_seed(seed);
 
   cscm_init();
 
   create_main_window ();
+  vbox = gtk_vbox_new (FALSE, 0);
+  gnome_app_set_contents (GNOME_APP (app), vbox);
 
   load_pixmaps ();
 
-  create_sol_board ();
   create_menus ();
 
+  gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);  
+
+  create_sol_board ();
+
   score_box = gtk_hbox_new(0, FALSE);
   score_label = gtk_label_new (_("Score:"));
   gtk_box_pack_start (GTK_BOX(score_box), score_label, FALSE, FALSE, 0);
@@ -416,15 +419,14 @@ static void main_prog(int argc, char *argv[])
   gtk_box_pack_end (GTK_BOX(status_bar), score_box, FALSE, FALSE, 0);
   gnome_app_set_statusbar (GNOME_APP (app), status_bar);
 
-  install_menu_hints(GNOME_APP (app));
-
-  new_game (start_game, &seed);
+  new_game (start_game, NULL);
 
   gtk_widget_show_all (app);
 
   if (!gconf_client_get_bool (gconf_client,
                              "/apps/aisleriot/show_toolbar", NULL))
-    toolbar_hide();
+    gtk_widget_hide (toolbar);
+
   click_to_move = gconf_client_get_bool (gconf_client, 
 					 "/apps/aisleriot/click_to_move", 
 					 NULL);
@@ -503,6 +505,6 @@ int main (int argc, char *argv [])
     g_free (var_file);
   }
 
-  gh_enter(argc, argv, main_prog);
+  scm_boot_guile (argc, argv, main_prog, NULL);
   return 0;
 }
