@@ -1,6 +1,8 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil -*-
-/* Blackjack - slot.cpp
- * Copyright (C) 2003 William Jon McCann <mccann@jhu.edu>
+/*
+ * Blackjack - slot.cpp
+ *
+ * Copyright (C) 2003-2004 William Jon McCann <mccann@jhu.edu>
  * Copyright (C) 1998 Jonathan Blandford <jrb@mit.edu>
  *
  * This game is free software; you can redistribute it and/or modify
@@ -20,6 +22,7 @@
  */
 
 #include <stdlib.h>
+#include <games-preimage.h>
 
 #include "blackjack.h"
 #include "slot.h"
@@ -27,9 +30,11 @@
 #include "chips.h"
 #include "draw.h"
 
+#define SLOT_FILENAME "white-outline.svg"
+
 GList *slot_list = NULL;
 
-GdkPixbuf *slot_pixbuf = NULL;
+GamesPreimage *slot_preimage = NULL;
 GdkPixbuf *slot_scaled_pixbuf = NULL;
 GdkPixmap *default_background_pixmap = NULL;
 
@@ -39,10 +44,29 @@ get_background_pixmap () {
         return default_background_pixmap;
 }
 
-GdkPixbuf *
-bj_slot_get_pixbuf ()
+void
+bj_slot_set_size (gint width,
+                  gint height)
 {
-        return slot_pixbuf;
+        gchar *name;
+
+        name = g_build_filename (PIXMAPDIR, "cards", "slots", SLOT_FILENAME, NULL);
+
+        if (!name)
+                return;
+
+        if (!slot_preimage)
+                slot_preimage = games_preimage_new_from_uri (name,
+                                                             NULL);
+
+        if (slot_scaled_pixbuf)
+                g_object_unref (slot_scaled_pixbuf);
+
+        slot_scaled_pixbuf = games_preimage_render (slot_preimage,
+                                                    width,
+                                                    height,
+                                                    NULL);
+        g_free (name);
 }
 
 GdkPixbuf *
@@ -51,24 +75,12 @@ bj_slot_get_scaled_pixbuf ()
         return slot_scaled_pixbuf;
 }
 
-void
-bj_slot_set_scaled_pixbuf (GdkPixbuf *pixbuf)
-{
-        if (slot_scaled_pixbuf)
-                g_object_unref (slot_scaled_pixbuf);
-
-        slot_scaled_pixbuf = pixbuf;
-}
-
 void
 bj_slot_load_pixmaps (void)
 {
         gchar *buffer;
 
-        buffer = g_build_filename ("cards", "slots", "white-outline.png", NULL);
-        slot_pixbuf = get_pixbuf (buffer);
-        g_free (buffer);
-        buffer = g_build_filename ("blackjack", "baize.png", NULL);
+        buffer = g_build_filename (PIXMAPDIR, "blackjack", "baize.png", NULL);
         default_background_pixmap = get_pixmap (buffer);
         g_free (buffer);
 }
@@ -76,9 +88,6 @@ bj_slot_load_pixmaps (void)
 void
 bj_slot_free_pixmaps (void)
 {
-        if (slot_pixbuf)
-                g_object_unref (slot_pixbuf);
-
         if (slot_scaled_pixbuf)
                 g_object_unref (slot_scaled_pixbuf);
 
